package com.supervision.service.impl;

import cn.hutool.core.lang.Assert;
import com.supervision.domain.IrKnowledge;
import com.supervision.domain.IrSessionParam;
import com.supervision.domain.IrSqlParam;
import com.supervision.service.IrKnowledgeService;
import com.supervision.service.IrSessionParamService;
import com.supervision.service.IrSqlParamService;
import com.supervision.service.QueryTemplateProcessor;
import com.supervision.util.ParamTypeConverter;
import com.supervision.util.freemark.StringTemplateConfig;
import com.supervision.util.mybatis.RowSqlMapper;
import freemarker.template.TemplateException;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

@Slf4j
@Service
@RequiredArgsConstructor
public class QueryTemplateProcessorImpl implements QueryTemplateProcessor {

    private final IrKnowledgeService knowledgeService;

    private final IrSqlParamService sqlParamService;

    private final IrSessionParamService sessionParamService;

    private final RowSqlMapper rowSqlMapper;
    @Override
    public String process(String templateId,String sessionId) throws TemplateException, IOException {

        Assert.notEmpty(templateId, "模板ID不能为空");

        // 根据模板ID查询查询sql模板和结果模板
        IrKnowledge knowledge = knowledgeService.getById(templateId);
        Assert.notNull(knowledge, "知识不存在！");
        Assert.notEmpty(knowledge.getSqlTemplate(), "模板内容不能为空");
        Assert.notEmpty(knowledge.getResultTemplate(), "结果模板不能为空");

        // 查询必填的参数
        List<IrSqlParam> paramsList = sqlParamService.lambdaQuery().eq(IrSqlParam::getKnowledgeId, templateId).eq(IrSqlParam::getParamRequire, 1).list();

        // 校验必填参数
        List<IrSessionParam> sessionParams = sessionParamService.lambdaQuery().eq(IrSessionParam::getSessionId, sessionId).list();
        Set<String> sessionParamsNames = sessionParams.stream().map(IrSessionParam::getParamName).collect(Collectors.toSet());
        paramsList.forEach(param -> Assert.isTrue(sessionParamsNames.contains(param.getParamName()), "参数["+param.getParamName()+"]未设置！"));

        // 执行查询sql
        ParamTypeConverter converter = ParamTypeConverter.getInstance();
        Map<String, Object> params = sessionParams.stream().collect(Collectors.toMap(IrSessionParam::getParamName, v->converter.convert(v.getParamType(),v.getParamValue()), (v1, v2) -> v1));
        List<Map<String, Object>> maps = rowSqlMapper.selectList(knowledge.getSqlTemplate(), params);

        // 组装返回结果
        return StringTemplateConfig.getInstance().process(knowledge.getResultTemplate(), maps);
    }



}
